@@ -30,7 +30,7 @@ use List::Compare;
 use List::MoreUtils qw(any none);
 use Path::Tiny;
 use Text::Levenshtein qw(distance);
-use XML::Simple qw(:strict);
+use XML::LibXML;
 
 use Lintian::Data;
 use Lintian::Deb822Parser qw(read_dpkg_control parse_dpkg_control);
@@ -52,9 +52,10 @@ use constant {
 use constant {
     WC_TYPE_REGEX => 'REGEX',
     WC_TYPE_FILE => 'FILE',
-    WC_TYPE_DECENDANTS => 'DECENDANTS',
+    WC_TYPE_DESCENDANTS => 'DESCENDANTS',
 };
 
+use constant EMPTY => q{};
 use constant SPACE => q{ };
 
 use Moo;
@@ -99,7 +100,7 @@ sub source {
     my $processable = $self->processable;
     my $group = $self->group;
 
-    my $debian_dir = $processable->index_resolved_path('debian/');
+    my $debian_dir = $processable->patched->resolve_path('debian/');
     return if not $debian_dir;
     my $copyright_path = $debian_dir->child('copyright');
 
@@ -119,7 +120,7 @@ sub source {
     }
 
     if ($copyright_path->is_open_ok) {
-        my $contents = $copyright_path->file_contents;
+        my $contents = $copyright_path->slurp;
 
         $self->check_dep5_copyright($contents);
         $self->check_apache_notice_files($contents);
@@ -191,27 +192,36 @@ sub find_dep5_version {
 sub check_apache_notice_files {
     my ($self, $contents) = @_;
 
-    my $processable = $self->processable;
-    my $group = $self->group;
+    return
+      unless $contents =~ m/apache[-\s]+2\./i;
 
-    my @procs = $group->get_processables('binary');
-    return if not @procs;
-    return if $contents !~ m/apache[-\s]+2\./i;
+    my $processable = $self->processable;
 
     my @notice_files = grep {
               $_->basename =~ m/^NOTICE(\.txt)?$/
           and $_->is_open_ok
-          and $_->file_contents =~ m/apache/i
-    } $processable->sorted_index;
-    return if not @notice_files;
+          and $_->slurp =~ m/apache/i
+    } $processable->patched->sorted_list;
+    return
+      unless @notice_files;
 
-    foreach my $binpkg (@procs) {
-        my @files = map { $_->name } $binpkg->sorted_index;
-        my $java_info = $binpkg->java_info;
-        for my $jar_file (sort keys %{$java_info}) {
-            push @files, keys %{$java_info->{$jar_file}{files}};
-        }
-        return if any { m{/NOTICE(\.txt)?(\.gz)?$} } @files;
+    my $group = $self->group;
+    my @binaries = $group->get_processables('binary');
+    return
+      unless @binaries;
+
+    foreach my $binary (@binaries) {
+
+        # look at all path names in the package
+        my @names = map { $_->name } $binary->installed->sorted_list;
+
+        # and also those shipped in jars
+        my @jars = grep { scalar keys %{$_->java_info} }
+          $binary->installed->sorted_list;
+        push(@names, keys %{$_->java_info->{files}})for @jars;
+
+        return
+          if any { m{/NOTICE(\.txt)?(\.gz)?$} } @names;
     }
 
     $self->tag('missing-notice-file-for-apache-license',
@@ -227,10 +237,17 @@ sub check_dep5_copyright {
 
     my (@dep5, @lines);
 
-    if (    $contents =~ m/^Files-Excluded:/
-        and $contents
-        !~ m{^Format:.*/doc/packaging-manuals/copyright-format/1.0$}) {
-        $self->tag('files-excluded-without-copyright-format-1.0');
+    if ($contents =~ /^Files-Excluded:/m) {
+
+        if ($contents
+            =~ m{^Format:.*/doc/packaging-manuals/copyright-format/1.0/?$}m) {
+
+            $self->tag('repackaged-source-not-advertised')
+              unless $processable->repacked || $processable->native;
+
+        } else {
+            $self->tag('files-excluded-without-copyright-format-1.0');
+        }
     }
 
     if (
@@ -325,7 +342,12 @@ sub parse_dep5 {
                 $field,$renamed_to, "(line $lines[0]{$field})");
         }
     }
-    $self->check_files_excluded($first_para->{'files-excluded'} // '');
+
+    $self->check_files_excluded($first_para->{'files-excluded'} // '')
+      unless $processable->native;
+
+    $self->tag('copyright-excludes-files-in-native-package')
+      if exists $first_para->{'files-excluded'} && $processable->native;
 
     $self->tag('missing-field-in-dep5-copyright',
         'format',"(line $lines[0]{'format'})")
@@ -353,17 +375,29 @@ sub parse_dep5 {
         }
     }
 
-    my @shippedfiles = sort grep { $_->is_file } $processable->sorted_index;
+    my @shipped;
+    if($processable->native) {
+        @shipped = $processable->patched->sorted_list;
+    } else {
+        @shipped = $processable->orig->sorted_list;
+    }
+
+    my $debian_dir = $processable->patched->resolve_path('debian/');
+    if ($debian_dir) {
+
+        push(@shipped, $debian_dir->descendants);
+    }
+
+    my @shippedfiles = sort grep { $_->is_file } @shipped;
 
     my @licensefiles= grep { m,(^|/)(COPYING[^/]*|LICENSE)$, } @shippedfiles;
-
     my @quiltfiles = grep { m,^\.pc/, } @shippedfiles;
 
     my (@commas_in_files, %file_para_coverage, %file_licenses);
     my %file_coverage = map { $_ => 0 } @shippedfiles;
     my $i = 0;
     my $current_line = 0;
-    my $commas_in_files = any { m/,/xsm } $processable->sorted_index;
+    my $commas_in_files = any { m/,/xsm } @shipped;
 
     for my $para (@dep5) {
         $i++;
@@ -465,29 +499,51 @@ sub parse_dep5 {
 
                     my $used = 0;
                     $file_para_coverage{$current_line} = 0;
+
                     if ($wc_type eq WC_TYPE_FILE) {
                         if (exists($file_coverage{$wc_value})) {
                             $used = 1;
                             $file_coverage{$wildcard} = $current_line;
                             $file_licenses{$wildcard} = $short_license;
                         }
-                    } elsif ($wc_type eq WC_TYPE_DECENDANTS) {
+
+                    } elsif ($wc_type eq WC_TYPE_DESCENDANTS) {
                         my @wlist;
-                        if (my $dir = $processable->index($wc_value)) {
-                            if ($wc_value eq q{}) {
-                                # Special-case => Files: *
-                                push(@wlist, @shippedfiles);
+
+                        if ($wc_value eq q{}) {
+                            # Special-case => Files: *
+                            push(@wlist, @shippedfiles);
+
+                        } elsif ($wc_value =~ /^debian\//) {
+                            my $dir = $processable->patched->lookup($wc_value);
+                            if ($dir) {
+                                my @files
+                                  = grep { $_->is_file }$dir->descendants;
+                                push(@wlist, @files);
+                            }
+
+                        } else {
+                            my $dir;
+                            if ($processable->native) {
+                                $dir= $processable->patched->lookup($wc_value);
                             } else {
-                                push(@wlist,
-                                    grep { $_->is_file }
-                                      $dir->children('breadth-first'));
+                                $dir = $processable->orig->lookup($wc_value);
+                            }
+                            if ($dir) {
+                                my @files
+                                  = grep { $_->is_file }$dir->descendants;
+                                push(@wlist, @files);
                             }
-                            $used = 1 if @wlist;
                         }
+
+                        $used = 1
+                          if @wlist;
+
                         for my $entry (@wlist) {
                             $file_coverage{$entry->name} = $current_line;
                             $file_licenses{$entry->name} = $short_license;
                         }
+
                     } else {
                         for my $srcfile (@shippedfiles) {
                             if ($srcfile =~ $wc_value) {
@@ -563,15 +619,32 @@ sub parse_dep5 {
             $lines[$paragraph_no]{$field_name});
     } else {
         foreach my $srcfile (sort keys %file_licenses) {
-            next if $srcfile =~ '^\.pc/';
-            next unless $srcfile =~ /\.xml$/;
-            my $file = $processable->index_resolved_path($srcfile);
-            my $seen = eval {
-                my $xml
-                  = XMLin($file->fs_path, ForceArray => [], KeyAttr => []);
-                lc($xml->{'metadata_license'} // '');
-            };
-            next unless $seen;
+            next
+              if $srcfile =~ '^\.pc/';
+            next
+              unless $srcfile =~ /\.xml$/;
+
+            my $parser = XML::LibXML->new;
+            $parser->set_option('no_network', 1);
+
+            my $file = $processable->patched->resolve_path($srcfile);
+            my $doc = eval {$parser->parse_file($file->unpacked_path);};
+            next
+              unless $doc;
+
+            my @nodes = $doc->findnodes('/component/metadata_license');
+            next
+              unless @nodes;
+
+            # take first one
+            my $first = $nodes[0];
+            next
+              unless $first;
+
+            my $seen = lc($first->firstChild->data // EMPTY);
+            next
+              unless $seen;
+
             my $wanted = $file_licenses{$srcfile};
             $self->tag('inconsistent-appstream-metadata-license',
                 $srcfile,"($seen != $wanted)")
@@ -594,6 +667,7 @@ sub parse_dep5 {
             "paragraph at line $_")
           for sort keys %file_para_coverage;
     }
+
     while ((my $license, $i) = each %required_standalone_licenses) {
         if (not defined $standalone_licenses{$license}) {
             $self->tag('missing-license-paragraph-in-dep5-copyright',
@@ -655,6 +729,11 @@ sub parse_license {
     }else {
         $short_license = $license_block;
     }
+    if(defined $full_license) {
+        $self->tag('tab-in-license-text',
+            "debian/copyright (paragraph at line $line)")
+          if $full_license =~ /\t/;
+    }
     $short_license =~ s/[(),]/ /g;
     if ($short_license =~ m/\A\s*\Z/) {
         $self->tag('empty-short-license-in-dep5-copyright',
@@ -718,7 +797,7 @@ sub parse_wildcard {
     $regex_src =~ s,^\./+,,;
     $regex_src =~ s,//+,/,g;
     if ($regex_src eq '*') {
-        return ('', WC_TYPE_DECENDANTS, undef);
+        return ('', WC_TYPE_DESCENDANTS, undef);
     }
     if (index($regex_src, '?') == -1) {
         my $star_index = index($regex_src, '*');
@@ -737,7 +816,7 @@ sub parse_wildcard {
             if ($bslash_index > -1) {
                 ($regex_src, $error) = dequote_backslashes($regex_src);
             }
-            return ($regex_src, WC_TYPE_DECENDANTS, $error);
+            return ($regex_src, WC_TYPE_DESCENDANTS, $error);
         }
     }
 
@@ -775,7 +854,7 @@ sub check_files_excluded {
 
     my $processable = $self->processable;
 
-    my @files = grep { $_->is_file } $processable->sorted_orig_index;
+    my @files = grep { $_->is_file } $processable->orig->sorted_list;
     my @wildcards = split /[\n\t ]+/, $excluded;
     for my $wildcard (@wildcards) {
         $wildcard =~ s/^\s+|\s+$//g;
@@ -827,12 +906,12 @@ sub binary {
     my $linked = 0;
     my $path = "usr/share/doc/$pkg";
 
-    if ($processable->index("$path/copyright.gz")) {
+    if ($processable->installed->lookup("$path/copyright.gz")) {
         $self->tag('copyright-file-compressed');
         $found = 1;
     }
 
-    if (my $index_info = $processable->index("$path/copyright")) {
+    if (my $index_info = $processable->installed->lookup("$path/copyright")) {
         $found = 1;
         if ($index_info->is_symlink) {
             $self->tag('copyright-file-is-symlink');
@@ -843,7 +922,7 @@ sub binary {
     }
 
     if (not $found) {
-        my $index_info = $processable->index($path);
+        my $index_info = $processable->installed->lookup($path);
         if (defined $index_info && $index_info->is_symlink) {
             my $link = $index_info->link;
 
@@ -1111,7 +1190,7 @@ qr/GNU (?:Lesser|Library) General Public License|(?-i:\bLGPL\b)/i
 
     check_spelling(
         $_,
-        $group->info->spelling_exceptions,
+        $group->spelling_exceptions,
         $self->spelling_tag_emitter('spelling-error-in-copyright'), 0
     );
 
