@@ -71,18 +71,18 @@ sub always {
 
     my $pkg = $self->package;
     my $type = $self->type;
-    my $info = $self->info;
+    my $processable = $self->processable;
 
     my ($seenconfig, $seentemplates, $usespreinst);
 
     if ($type eq 'source') {
-        my @binaries = $info->binaries;
+        my @binaries = $processable->binaries;
         my @files = map { "$_.templates" } @binaries;
         push @files, 'templates';
 
         foreach my $file (@files) {
             my $dfile = "debian/$file";
-            my $templates_file = $info->index_resolved_path($dfile);
+            my $templates_file = $processable->patched->resolve_path($dfile);
             my $binary = $file;
             $binary =~ s/\.?templates$//;
             # Single binary package (so @files contains "templates" and
@@ -95,7 +95,7 @@ sub always {
                 my @templates;
                 eval {
                     @templates
-                      = read_dpkg_control($templates_file->fs_path,
+                      = read_dpkg_control($templates_file->unpacked_path,
                         DCTRL_DEBCONF_TEMPLATE);
                 };
                 if ($@) {
@@ -122,12 +122,12 @@ sub always {
         return;
     }
 
-    my $preinst = $info->control_index('preinst');
-    my $ctrl_config = $info->control_index('config');
-    my $ctrl_templates = $info->control_index('templates');
+    my $preinst = $processable->control->lookup('preinst');
+    my $ctrl_config = $processable->control->lookup('config');
+    my $ctrl_templates = $processable->control->lookup('templates');
 
     if ($preinst and $preinst->is_file and $preinst->is_open_ok) {
-        my $fd = $preinst->open;
+        open(my $fd, '<', $preinst->unpacked_path);
         while (<$fd>) {
             s/\#.*//;    # Not perfect for Perl, but should be OK
             if (   m,/usr/share/debconf/confmodule,
@@ -152,8 +152,8 @@ sub always {
 
     # Consider every package to depend on itself.
     my $selfrel;
-    if (defined $info->field('version')) {
-        $_ = $info->field('version');
+    if (defined $processable->field('version')) {
+        $_ = $processable->field('version');
         $selfrel = "$pkg (= $_)";
     } else {
         $selfrel = "$pkg";
@@ -162,9 +162,9 @@ sub always {
     # Include self and provides as a package providing debconf presumably
     # satisfies its own use of debconf (if any).
     my $selfrelation
-      = Lintian::Relation->and($info->relation('provides'), $selfrel);
+      = Lintian::Relation->and($processable->relation('provides'), $selfrel);
     my $alldependencies
-      = Lintian::Relation->and($info->relation('strong'), $selfrelation);
+      = Lintian::Relation->and($processable->relation('strong'),$selfrelation);
 
     # See if the package depends on dbconfig-common.  Packages that do
     # are allowed to have a config file with no templates, since they
@@ -194,8 +194,7 @@ sub always {
             # $seentemplates (above) will be false if $ctrl_templates is a
             # symlink or not a file, so this should be safe without
             # (re-checking) with -f/-l.
-            @templates
-              = read_dpkg_control($ctrl_templates->fs_path,
+            @templates= read_dpkg_control($ctrl_templates->unpacked_path,
                 DCTRL_DEBCONF_TEMPLATE);
         };
         if ($@) {
@@ -401,11 +400,11 @@ sub always {
     my (%templates_used, %template_aliases);
     for my $file (qw(config prerm postrm preinst postinst)) {
         my $potential_makedev = {};
-        my $path = $info->control_index($file);
+        my $path = $processable->control->lookup($file);
         if ($path and $path->is_file and $path->is_open_ok) {
             my ($usesconfmodule, $obsoleteconfmodule, $db_input, $isdefault);
 
-            my $fd = $path->open;
+            open(my $fd, '<', $path->unpacked_path);
             # Only check scripts.
             my $fl = <$fd>;
             unless ($fl && $fl =~ /^\#!/) {
@@ -455,7 +454,8 @@ sub always {
                      [\"\']? (\S+?) [\"\']? \s+ (\S+)\s/xsm
                 ) {
                     my ($priority, $template) = ($1, $2);
-                    $templates_used{get_template_name($info, $template)} = 1;
+                    $templates_used{get_template_name($processable, $template)}
+                      = 1;
                     if ($priority !~ /^\$\S+$/) {
                         $self->tag('unknown-debconf-priority', "$file:$. $1")
                           unless ($valid_priorities{$priority});
@@ -473,7 +473,7 @@ sub always {
                     m/ \A \s* (?:db_get|db_set(?:title)?) \s+ 
                        [\"\']? (\S+?) [\"\']? (?:\s|\Z)/xsm
                 ) {
-                    $templates_used{get_template_name($info, $1)} = 1;
+                    $templates_used{get_template_name($processable, $1)} = 1;
                 }
                 # Try to handle Perl somewhat.
                 if (   m/^\s*(?:.*=\s*get|set)\s*\(\s*[\"\'](\S+?)[\"\']/
@@ -552,7 +552,7 @@ sub always {
     # package that might provide debconf functionality.
 
     if ($usespreinst) {
-        unless ($info->relation('pre-depends')->implies($ANY_DEBCONF)) {
+        unless ($processable->relation('pre-depends')->implies($ANY_DEBCONF)) {
             $self->tag('missing-debconf-dependency-for-preinst')
               unless $type eq 'udeb';
         }
@@ -568,17 +568,21 @@ sub always {
     # the following checks is ignored if the package being checked is debconf
     # itself.
 
-    return if ($pkg eq 'debconf') || ($type eq 'udeb');
+    return
+      if ($pkg eq 'debconf') || ($type eq 'udeb');
 
-    foreach my $filename (sort keys %{$info->scripts}) {
-        my $path = $info->index_resolved_path($filename);
-        next if not $path or not $path->is_open_ok;
-        my $fd = $path->open;
+    my @scripts = grep { $_->is_script } $processable->installed->sorted_list;
+    foreach my $file (@scripts) {
+
+        next
+          unless $file->is_open_ok;
+
+        open(my $fd, '<', $file->unpacked_path);
         while (<$fd>) {
             s/#.*//;    # Not perfect for Perl, but should be OK
             if (   m,/usr/share/debconf/confmodule,
                 or m/(?:Debconf|Debian::DebConf)::Client::ConfModule/) {
-                $self->tag('debconf-is-not-a-registry', $filename);
+                $self->tag('debconf-is-not-a-registry', $file->name);
                 last;
             }
         }
@@ -612,9 +616,10 @@ sub count_choices {
 
 # Manually interpolate shell variables, eg. $DPKG_MAINTSCRIPT_PACKAGE
 sub get_template_name {
-    my ($info, $name) = @_;
+    my ($processable, $name) = @_;
 
-    return $name =~ s/^\$DPKG_MAINTSCRIPT_PACKAGE/$info->{name}/r;
+    my $package = $processable->name;
+    return $name =~ s/^\$DPKG_MAINTSCRIPT_PACKAGE/$package/r;
 }
 
 1;
