@@ -44,9 +44,8 @@ use Lintian::Inspect::Changelog;
 use Lintian::Internal::FrontendUtil
   qw(default_parallel sanitize_environment open_file_or_fd);
 use Lintian::Output::Standard;
-use Lintian::Processable::Pool;
+use Lintian::Pool;
 use Lintian::Profile;
-use Lintian::Tags;
 use Lintian::Util qw(internal_error parse_boolean strip safe_qx);
 
 # only in GNOME; need original environment
@@ -89,7 +88,7 @@ my %opt = (                     #hash of some flags from cmd or cfg
 
 my $experimental_output_opts;
 
-my (@CLOSE_AT_END, $TAGS);
+my @CLOSE_AT_END;
 my $OUTPUT = Lintian::Output::Standard->new;
 my @certainties = qw(wild-guess possible certain);
 my (@display_level, %display_source, %suppress_tags);
@@ -99,8 +98,6 @@ my $user_dirs = $ENV{'LINTIAN_ENABLE_USER_DIRS'} // 1;
 my $exit_code = 0;
 my $STATUS_FD;
 
-my $memory_usage = sub { 'N/A'; };
-
 # }}}
 
 # {{{ Setup Code
@@ -383,6 +380,11 @@ sub display_pedantictags {
     return;
 }
 
+sub display_classificationtags {
+    push(@display_level, ['+', '=', 'classification']);
+    return;
+}
+
 # Process --default-display-level flag
 sub default_display_level {
     push(@display_level,
@@ -570,6 +572,8 @@ sub _main {
 
 sub main {
 
+    $0 = join(' ', $0, @ARGV);
+
     #turn off file buffering
     STDOUT->autoflush;
     binmode(STDOUT, ':utf8');
@@ -657,41 +661,9 @@ sub main {
         $opt{'verbose'} = 1;
         $ENV{'LINTIAN_DEBUG'} = $opt{'debug'};
         $SIG{__DIE__} = sub { Carp::confess(@_) };
-        if ($opt{'debug'} > 2) {
-            eval {
-                require Devel::Size;
-                Devel::Size->import(qw(total_size));
-                {
-                    no warnings qw(once);
-                    # Disable warnings about stuff Devel::Size cannot
-                    # give reliable sizes for.
-                    $Devel::Size::warn = 0;
-                }
-
-                $memory_usage = sub {
-                    my ($obj) = @_;
-                    my $size = total_size($obj);
-                    my $unit = 'B';
-                    if ($size > 1536) {
-                        $size /= 1024;
-                        $unit = 'kB';
-                        if ($size > 1536) {
-                            $size /= 1024;
-                            $unit = 'MB';
-                        }
-                    }
-                    return sprintf('%.2f %s', $size, $unit);
-                };
-                print "N: Using Devel::Size to debug memory usage\n";
-            };
-            if ($@) {
-                print "N: Cannot load Devel::Size ($@)\n";
-                print "N: Running memory usage will not be checked.\n";
-            }
-        }
     } else {
         # Ensure verbose has a defined value
-        $opt{'verbose'} = 0 unless defined($opt{'verbose'});
+        $opt{'verbose'} //= 0;
     }
 
     $OUTPUT->verbosity_level($opt{'verbose'});
@@ -757,19 +729,17 @@ sub main {
     $OUTPUT->v_msg('Using profile ' . $PROFILE->name . '.');
     Lintian::Data->set_vendor($PROFILE);
 
-    $TAGS = Lintian::Tags->new;
-    $TAGS->show_experimental($opt{'display-experimental'});
-    $TAGS->show_overrides($opt{'show-overrides'});
-    $TAGS->sources(keys(%display_source)) if %display_source;
-    $TAGS->profile($PROFILE);
+    $PROFILE->show_experimental($opt{'display-experimental'});
+    $PROFILE->sources(keys %display_source)
+      if %display_source;
 
     if ($dont_check || %suppress_tags || $checks || $check_tags) {
-        _update_profile($PROFILE, $TAGS, $dont_check, \%suppress_tags,$checks);
+        _update_profile($PROFILE, $dont_check, \%suppress_tags,$checks);
     }
 
     # Initialize display level settings.
     for my $level (@display_level) {
-        eval { $TAGS->display(@{$level}) };
+        eval { $PROFILE->display(@{$level}) };
         fatal_error($@) if $@;
     }
 
@@ -807,14 +777,15 @@ sub main {
           unless fileno($fd) == fileno(STDIN);
     }
 
-    my $pool = Lintian::Processable::Pool->new;
+    my $pool = Lintian::Pool->new;
 
     for my $path (@subjects) {
 
+        # in ubuntu, automatic dbgsym packages end with .ddeb
         fatal_error
-"bad package file name $path (neither .deb, .udeb, .changes, .dsc or .buildinfo file)"
+"bad package file name $path (neither .deb, .udeb, .ddeb, .changes, .dsc or .buildinfo file)"
           unless -f $path
-          && $path =~ m/\.(?:u?deb|dsc|changes|buildinfo)$/;
+          && $path =~ m/\.(?:[u|d]?deb|dsc|changes|buildinfo)$/;
 
         my $absolute = Cwd::abs_path($path);
         die "Cannot resolve $path: $!"
@@ -822,7 +793,7 @@ sub main {
 
         eval {
             # create a new group
-            my $group = Lintian::Processable::Group->new;
+            my $group = Lintian::Group->new;
             $group->pooldir($pool->basedir);
             $group->init_from_file($absolute);
 
@@ -841,10 +812,8 @@ sub main {
 
     $ENV{INIT_ROOT} = $INIT_ROOT;
 
-    $pool->process(
-        $action, $PROFILE,$TAGS,\$exit_code, \%opt,
-        $memory_usage,$STATUS_FD, \@unpack_info, $OUTPUT
-    );
+    $pool->process($action, $PROFILE,\$exit_code, \%opt,
+        $STATUS_FD, \@unpack_info, $OUTPUT);
 
     retrigger_signal()
       if $received_signal;
@@ -1100,17 +1069,7 @@ sub parse_options {
 }
 
 sub _update_profile {
-    my ($profile, $tags, $sup_check, $sup_tags, $only_check) = @_;
-    my %abbrev = ();
-
-    if ($sup_check || $only_check) {
-        # Build an abbreviation map
-        for my $c ($profile->scripts(1)) {
-            my $cs = $profile->get_script($c, 1);
-            next unless $cs->abbrev;
-            $abbrev{$cs->abbrev} = $cs;
-        }
-    }
+    my ($profile, $sup_check, $sup_tags, $only_check) = @_;
 
     # if tags are listed explicitly (--tags) then show them even if
     # they are pedantic/experimental etc.  However, for --check-part
@@ -1118,12 +1077,13 @@ sub _update_profile {
     if ($checks || $check_tags) {
         $profile->disable_tags($profile->tags);
         if ($check_tags) {
-            $tags->show_experimental(1);
+            $profile->show_experimental(1);
             # discard whatever is in @display_level and request
             # everything
             @display_level = ();
             display_infotags();
             display_pedantictags();
+            display_classificationtags();
             $profile->enable_tags(split /,/, $check_tags);
         } else {
             for my $c (split /,/, $checks) {
@@ -1133,7 +1093,7 @@ sub _update_profile {
                     $profile->enable_tags($_->tags)for @all;
                     next;
                 }
-                my $cs = $profile->get_script($c, 1) || $abbrev{$c};
+                my $cs = $profile->get_script($c, 1);
                 fatal_error("Unrecognized check script (via -C): $c")
                   unless $cs;
                 $profile->enable_tags($cs->tags);
@@ -1142,7 +1102,7 @@ sub _update_profile {
     } elsif ($sup_check) {
         # we are disabling checks
         for my $c (split(/,/, $sup_check)) {
-            my $cs = $profile->get_script($c, 1) || $abbrev{$c};
+            my $cs = $profile->get_script($c, 1);
             fatal_error("Unrecognized check script (via -X): $c") unless $cs;
             $profile->disable_tags($cs->tags);
         }
