@@ -85,9 +85,9 @@ sub issue_tags {
             $self->delimiter,
             'Processing '. $processable->type. " $object ". $processable->name,
             '(version '
-              . $processable->pkg_version
+              . $processable->version
               . ', arch '
-              . $processable->pkg_arch . ') ...'
+              . $processable->architecture . ') ...'
         );
 
         my @subset = @{$taglist{$processable} // []};
