@@ -33,11 +33,21 @@ use constant SLASH => q{/};
 use Moo;
 use namespace::clean;
 
-with 'Lintian::Collect::Binary', 'Lintian::Info::Changelog',
-  'Lintian::Info::Control::Index', 'Lintian::Info::Control::Conffiles',
-  'Lintian::Info::Hardening', 'Lintian::Info::Objdump',
-  'Lintian::Info::Overrides', 'Lintian::Info::Package',
-  'Lintian::Info::Scripts', 'Lintian::Processable';
+with
+  'Lintian::Processable::Binary::Changelog',
+  'Lintian::Processable::Binary::Class',
+  'Lintian::Processable::Binary::Copyright',
+  'Lintian::Processable::Binary::Relation',
+  'Lintian::Processable::Changelog',
+  'Lintian::Processable::Control',
+  'Lintian::Processable::Control::Conffiles',
+  'Lintian::Processable::Installable',
+  'Lintian::Processable::Installed',
+  'Lintian::Processable::IsNonFree',
+  'Lintian::Processable::Hardening',
+  'Lintian::Processable::Objdump',
+  'Lintian::Processable::Overrides',
+  'Lintian::Processable';
 
 =for Pod::Coverage BUILDARGS
 
@@ -56,7 +66,7 @@ Lintian::Processable::Udeb -- A udeb installation package Lintian can process
 
 This class represents a 'udeb' file that Lintian can process. Objects
 
-of this kind are often part of a L<Lintian::Processable::Group>, which
+of this kind are often part of a L<Lintian::Group>, which
 represents all the files in a changes or buildinfo file.
 
 =head1 INSTANCE METHODS
