@@ -1,4 +1,5 @@
-# Copyright (C) 2011 Niels Thykier <niels@thykier.net>
+# Copyright © 2011 Niels Thykier <niels@thykier.net>
+# Copyright © 2020 Felix Lechner <felix.lechner@lease-up.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,26 +17,50 @@
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA.
 
-## Represents a Lintian profile
 package Lintian::Profile;
 
-use parent qw(Class::Accessor::Fast);
-
 use strict;
 use warnings;
 use autodie qw(opendir closedir);
 
 use Carp qw(croak);
 use File::Find::Rule;
+use List::MoreUtils qw(any);
 use Path::Tiny;
 
 use Dpkg::Vendor qw(get_current_vendor get_vendor_info);
 
 use Lintian::CheckScript;
 use Lintian::Deb822Parser qw(read_dpkg_control_utf8);
-use Lintian::Tags;
+use Lintian::Tag::Info;
 use Lintian::Util qw(parse_boolean strip);
 
+use constant EMPTY => q{};
+use constant SPACE => q{ };
+
+use Moo;
+use namespace::clean;
+
+# map of known valid severity allowed by profiles
+my %SEVERITIES = map { $_ => 1} @Lintian::Tag::Info::SEVERITIES;
+
+# List of fields in the main profile paragraph
+my %MAIN_FIELDS = (
+    'profile'                 => 1,
+    'extends'                 => 1,
+    'enable-tags-from-check'  => 1,
+    'disable-tags-from-check' => 1,
+    'enable-tags'             => 1,
+    'disable-tags'            => 1,
+);
+
+# List of fields in secondary profile paragraphs
+my %SEC_FIELDS = (
+    'tags'        => 1,
+    'overridable' => 1,
+    'severity'    => 1,
+);
+
 =head1 NAME
 
 Lintian::Profile - Profile parser for Lintian
@@ -59,37 +84,136 @@ Lintian::Profile - Profile parser for Lintian
 Lintian::Profile handles finding, parsing and implementation of
 Lintian Profiles as well as loading the relevant Lintian checks.
 
-=head1 CLASS METHODS
+=head1 INSTANCE METHODS
 
 =over 4
 
+=item $prof->aliases()
+
+Returns a hash with old names that have new names.
+
+=item $prof->profile_list
+
+Returns a list ref of the (normalized) names of the profile and its
+parents.  The last element of the list is the name of the profile
+itself, the second last is its parent and so on.
+
+Note: This list reference and its contents should not be modified.
+
+=item show_experimental(BOOL)
+
+If BOOL is true, configure experimental tags to be shown.  If BOOL is
+false, configure experimental tags to not be shown.
+
+=item $prof->name
+
+Returns the name of the profile, which may differ from the name used
+to create this instance of the profile (e.g. due to symlinks).
+
 =cut
 
-# map of known valid severity allowed by profiles
-my %SEVERITIES = map { $_ => 1} @Lintian::Tags::SEVERITIES;
+has aliases => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
 
-# List of fields in the main profile paragraph
-my %MAIN_FIELDS = (
-    'profile'                 => 1,
-    'extends'                 => 1,
-    'enable-tags-from-check'  => 1,
-    'disable-tags-from-check' => 1,
-    'enable-tags'             => 1,
-    'disable-tags'            => 1,
+has check_scripts => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has check_tagnames => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has display_level => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub {
+        {
+            classification =>
+              { 'wild-guess' => 0, possible => 0, certain => 0 },
+            wishlist  => { 'wild-guess' => 0, possible => 0, certain => 0 },
+            minor     => { 'wild-guess' => 0, possible => 0, certain => 1 },
+            normal    => { 'wild-guess' => 0, possible => 1, certain => 1 },
+            important => { 'wild-guess' => 1, possible => 1, certain => 1 },
+            serious   => { 'wild-guess' => 1, possible => 1, certain => 1 },
+        }
+    });
+
+has display_source => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has enabled_checks => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has enabled_tags => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has files => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has known_tags => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has name => (
+    is => 'rw',
+    coerce => sub { my ($string) = @_; return $string // EMPTY;},
+    default => EMPTY
 );
 
-# List of fields in secondary profile paragraphs
-my %SEC_FIELDS = (
-    'tags'        => 1,
-    'overridable' => 1,
-    'severity'    => 1,
+has non_overridable_tags => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has parent_map => (
+    is => 'rw',
+    coerce => sub { my ($hashref) = @_; return ($hashref // {}); },
+    default => sub { {} });
+
+has profile_list => (
+    is => 'rw',
+    coerce => sub { my ($arrayref) = @_; return ($arrayref // []); },
+    default => sub { [] });
+
+has show_experimental => (
+    is => 'rw',
+    coerce => sub { my ($boolean) = @_; return $boolean ? 1 : 0; },
+    default => 0
 );
 
-=item Lintian::Profile->new ([$profname[, $ipath[, $extra]]])
+has saved_include_path => (
+    is => 'rw',
+    coerce => sub { my ($arrayref) = @_; return ($arrayref // []); },
+    default => sub { [] });
 
-Creates a new profile from the profile.  $profname is the name of the
-profile and $ipath is a list reference containing the path
-to one (or more) Lintian "roots".
+has saved_safe_include_path => (
+    is => 'rw',
+    coerce => sub { my ($arrayref) = @_; return ($arrayref // []); },
+    default => sub { [] });
+
+has vendor_cache => (
+    is => 'rw',
+    coerce => sub { my ($arrayref) = @_; return ($arrayref // []); },
+    default => sub { [] });
+
+=item load ([$profname[, $ipath[, $extra]]])
+
+Loads a new profile.  $profname is the name of the profile and $ipath
+is a list reference containing the path to one (or more) Lintian
+"roots".
 
 If $profname is C<undef>, the default vendor will be loaded based on
 Dpkg::Vendor::get_current_vendor.
@@ -98,9 +222,11 @@ If $ipath is not given, a default one will be used.
 
 =cut
 
-sub new {
-    my ($type, $name, $ipath, $extra) = @_;
+sub load {
+    my ($self, $name, $ipath, $extra) = @_;
+
     my ($profile, @full_inc_path);
+
     if (!defined $ipath) {
         # Temporary fix (see _safe_include_path)
         @full_inc_path = (_default_inc_path());
@@ -118,34 +244,18 @@ sub new {
     }
     push @full_inc_path, @$ipath;
 
-    my $self = {
-        'parent-map'           => {},
-        'profile_list'         => [],
-        'include-path'         => \@full_inc_path,
-        'safe-include-path'    => $ipath,
-        # "set" of tags enabled (value is largely ignored)
-        'enabled-tags'         => {},
-        # maps script to the number of tags enabled (0 if disabled)
-        'enabled-checks'       => {},
-        'non-overridable-tags' => {},
-        # maps script name to Lintian::CheckScript
-        'check-scripts'        => {},
-        # maps script name to an array of tag names;
-        'check-tagnames'       => {},
-        # maps tag name to Lintian::Tag::Info
-        'known-tags'           => {},
-        # maps old tag names to new tag names
-        'renamed-tags'         => {},
-    };
-    $self = bless $self, $type;
-    if (not defined $name) {
-        ($profile, $name) = $self->_find_vendor_profile;
-    } else {
-        croak "Illegal profile name \"$name\""
-          if $name =~ m,^/,o
-          or $name =~ m/\./o;
+    $self->saved_include_path(\@full_inc_path);
+    $self->saved_safe_include_path($ipath);
+
+    if (defined $name) {
+        croak "Illegal profile name $name"
+          if $name =~ m{^/}
+          || $name =~ m{\.};
         ($profile, undef) = $self->_find_vendor_profile($name);
+    } else {
+        ($profile, $name) = $self->_find_vendor_profile;
     }
+
     croak "Cannot find profile $name (in "
       . join(', ', map { "$_/profiles" } @$ipath).')'
       unless $profile;
@@ -153,7 +263,8 @@ sub new {
     # populate known tags and their check associations
     for my $tagroot ($self->_safe_include_path('tags')) {
 
-        next unless -d $tagroot;
+        next
+          unless -d $tagroot;
 
         my @descfiles = File::Find::Rule->file()->name('*.desc')->in($tagroot);
         for my $tagpath (@descfiles) {
@@ -164,21 +275,21 @@ sub new {
             die "Tag in $tagpath is not associated with a check"
               unless length $taginfo->script;
 
-            unless (exists $self->{'known-tags'}{$taginfo->tag}) {
-                $self->{'known-tags'}{$taginfo->tag} = $taginfo;
+            unless (exists $self->known_tags->{$taginfo->tag}) {
+                $self->known_tags->{$taginfo->tag} = $taginfo;
                 push(
-                    @{$self->{'check-tagnames'}{$taginfo->script}},
+                    @{$self->check_tagnames->{$taginfo->script}},
                     $taginfo->tag
                 );
             }
 
             for my $alias ($taginfo->aliases) {
-                my $taken = $self->{'renamed-tags'}{$alias};
+                my $taken = $self->aliases->{$alias};
                 die "Internal error: tags $taken and "
                   . $taginfo->tag
                   . " share same alias $alias."
                   if defined $taken;
-                $self->{'renamed-tags'}{$alias} = $taginfo->tag;
+                $self->aliases->{$alias} = $taginfo->tag;
             }
         }
     }
@@ -187,7 +298,7 @@ sub new {
 
     # Implementation detail: Ensure that the "lintian" check is always
     # loaded to avoid "attempt to emit unknown tags" caused by
-    # the frontend or L::Tags.  Also default to enabling the Lintian
+    # the frontend.  Also default to enabling the Lintian
     # tags as they are helpful (e.g. for debugging overrides files)
     my $c = $self->_load_check($self->name, 'lintian');
     $self->enable_tags($c->tags);
@@ -196,23 +307,6 @@ sub new {
     return $self;
 }
 
-=item $prof->profile_list
-
-Returns a list ref of the (normalized) names of the profile and its
-parents.  The last element of the list is the name of the profile
-itself, the second last is its parent and so on.
-
-Note: This list reference and its contents should not be modified.
-
-=item $prof->name
-
-Returns the name of the profile, which may differ from the name used
-to create this instance of the profile (e.g. due to symlinks).
-
-=cut
-
-Lintian::Profile->mk_ro_accessors(qw(profile_list name));
-
 =item $prof->tags([$known])
 
 Returns the list of tags in this profile.  If $known is given
@@ -225,19 +319,11 @@ Note: The contents of this list should not be modified.
 
 sub tags {
     my ($self, $known) = @_;
-    return keys %{ $self->{'known-tags'} } if $known;
-    return keys %{ $self->{'enabled-tags'} };
-}
 
-=item $prof->aliases()
+    return keys %{ $self->known_tags }
+      if $known;
 
-Returns a hash with old names that have new names.
-
-=cut
-
-sub aliases {
-    my ($self) = @_;
-    return $self->{'renamed-tags'};
+    return keys %{ $self->enabled_tags };
 }
 
 =item $prof->scripts ([$known])
@@ -251,8 +337,11 @@ enabled.
 
 sub scripts {
     my ($self, $known) = @_;
-    return keys %{ $self->{'check-scripts'} } if $known;
-    return keys %{ $self->{'enabled-checks'} };
+
+    return keys %{ $self->check_scripts }
+      if $known;
+
+    return keys %{ $self->enabled_checks };
 }
 
 =item $prof->is_overridable ($tag)
@@ -264,7 +353,8 @@ Returns a false value if the tag has been marked as
 
 sub is_overridable {
     my ($self, $tag) = @_;
-    return !exists $self->{'non-overridable-tags'}{$tag};
+
+    return !exists $self->non_overridable_tags->{$tag};
 }
 
 =item $prof->get_tag ($tag[, $known])
@@ -277,8 +367,11 @@ Otherwise it returns undef.
 
 sub get_tag {
     my ($self, $tag, $known) = @_;
-    return unless $known || exists $self->{'enabled-tags'}{$tag};
-    return $self->{'known-tags'}{$tag};
+
+    return
+      unless $known || exists $self->enabled_tags->{$tag};
+
+    return $self->known_tags->{$tag};
 }
 
 =item $prof->get_script ($script[, $known])
@@ -294,8 +387,11 @@ provides are enabled.
 
 sub get_script {
     my ($self, $script, $known) = @_;
-    return unless $known || exists $self->{'enabled-checks'}{$script};
-    return $self->{'check-scripts'}{$script};
+
+    return
+      unless $known || exists $self->enabled_checks->{$script};
+
+    return $self->check_scripts->{$script};
 }
 
 =item $prof->enable_tags (@tags)
@@ -306,13 +402,19 @@ Enables all tags named in @tags.  Croaks if an unknown tag is found.
 
 sub enable_tags {
     my ($self, @tags) = @_;
+
     for my $tag (@tags) {
-        my $ti = $self->{'known-tags'}{$tag};
-        die "Unknown tag $tag" unless $ti;
-        next if exists $self->{'enabled-tags'}{$tag};
-        $self->{'enabled-tags'}{$tag} = 1;
-        $self->{'enabled-checks'}{$ti->script}++;
+        my $taginfo = $self->known_tags->{$tag};
+        die "Unknown tag $tag"
+          unless $taginfo;
+
+        next
+          if exists $self->enabled_tags->{$tag};
+
+        $self->enabled_tags->{$tag} = 1;
+        $self->enabled_checks->{$taginfo->script}++;
     }
+
     return;
 }
 
@@ -324,14 +426,20 @@ Disable all tags named in @tags.  Croaks if an unknown tag is found.
 
 sub disable_tags {
     my ($self, @tags) = @_;
+
     for my $tag (@tags) {
-        my $ti = $self->{'known-tags'}{$tag};
-        die "Unknown tag $tag" unless $ti;
-        next unless exists $self->{'enabled-tags'}{$tag};
-        delete $self->{'enabled-tags'}{$tag};
-        delete $self->{'enabled-checks'}{$ti->script}
-          unless --$self->{'enabled-checks'}{$ti->script};
+        my $taginfo = $self->known_tags->{$tag};
+        die "Unknown tag $tag"
+          unless $taginfo;
+
+        next
+          unless exists $self->enabled_tags->{$tag};
+
+        delete $self->enabled_tags->{$tag};
+        delete $self->enabled_checks->{$taginfo->script}
+          unless --$self->enabled_checks->{$taginfo->script};
     }
+
     return;
 }
 
@@ -351,10 +459,11 @@ Paths returned are not guaranteed to exists.
 
 sub include_path {
     my ($self, $path) = @_;
-    unless (defined $path) {
-        return @{ $self->{'include-path'} };
-    }
-    return map { "$_/$path" } @{ $self->{'include-path'} };
+
+    return map { "$_/$path" } @{ $self->saved_include_path }
+      if defined $path;
+
+    return @{ $self->saved_include_path };
 }
 
 # Temporary until aptdaemon (etc.) has been upgraded to handle
@@ -362,10 +471,11 @@ sub include_path {
 # LP: #1162947
 sub _safe_include_path {
     my ($self, $path) = @_;
-    unless (defined $path) {
-        return @{ $self->{'safe-include-path'} };
-    }
-    return map { "$_/$path" } @{ $self->{'safe-include-path'} };
+
+    return map { "$_/$path" } @{ $self->saved_safe_include_path }
+      if defined $path;
+
+    return @{ $self->saved_safe_include_path };
 }
 
 # $prof->_find_profile ($pname)
@@ -379,15 +489,21 @@ sub _safe_include_path {
 
 sub _find_profile {
     my ($self, $pname) = @_;
-    my $pfile;
-    croak "\"$pname\" is not a valid profile name" if $pname =~ m/\./o;
+
+    croak "$pname is not a valid profile name"
+      if $pname =~ m{\.};
+
     # $vendor is short for $vendor/main
-    $pname = "$pname/main" unless $pname =~ m,/,o;
-    $pfile = "$pname.profile";
+    $pname = "$pname/main"
+      unless $pname =~ m{/};
+
+    my $pfile = "$pname.profile";
     foreach my $path ($self->include_path('profiles')) {
-        return "$path/$pfile" if -e "$path/$pfile";
+        return "$path/$pfile"
+          if -e "$path/$pfile";
     }
-    return '';
+
+    return EMPTY;
 }
 
 # $self->_read_profile($pfile)
@@ -396,41 +512,48 @@ sub _find_profile {
 # normally, the profile will have been parsed successfully.
 sub _read_profile {
     my ($self, $pfile) = @_;
-    my $pheader;
-    my $pmap = $self->{'parent-map'};
-    my $pname;
-    my $plist = $self->{'profile_list'};
+
+    my $pmap = $self->parent_map;
+    my $plist = $self->profile_list;
+
     my @dirty = read_dpkg_control_utf8($pfile, 0);
     my @pdata = _clean_fields(@dirty);
-    $pheader = shift @pdata;
+
+    my $pheader = shift @pdata;
     croak "Profile field is missing from $pfile"
       unless defined $pheader && $pheader->{'profile'};
-    $pname = $pheader->{'profile'};
+
+    my $pname = $pheader->{'profile'};
     croak "Invalid Profile field in $pfile"
-      if $pname =~ m,^/,o
-      or $pname =~ m/\./o;
+      if $pname =~ m{^/} || $pname =~ m{\.};
 
     # Normalize the profile name
-    $pname .= '/main' unless $pname =~m,/,;
+    $pname .= '/main'
+      unless $pname =~ m{/};
 
     croak "Recursive definition of $pname"
       if exists $pmap->{$pname};
+
     $pmap->{$pname} = 0; # Mark as being loaded.
-    $self->{'name'} = $pname unless exists $self->{'name'};
-    if (exists $pheader->{'extends'}){
-        my $parent = $pheader->{'extends'};
-        my $parentf;
+
+    $self->name($pname)
+      unless length $self->name;
+
+    my $parent = $pheader->{'extends'};
+    if (length $parent){
         croak "Invalid Extends field in $pfile"
-          unless $parent && $parent !~ m/\./o;
-        ($parentf, undef) = $self->_find_vendor_profile($parent);
+          if $parent =~ m{\.};
+
+        my ($parentf, undef) = $self->_find_vendor_profile($parent);
         croak "Cannot find $parent, which $pname extends"
           unless $parentf;
+
         $self->_read_profile($parentf);
     }
 
     # Add the profile to the "chain" after loading its parent (if
     # any).
-    push @$plist, $pname;
+    push(@$plist, $pname);
 
     $self->_read_profile_tags($pname, $pheader);
     if (@pdata){
@@ -439,6 +562,7 @@ sub _read_profile {
             $self->_read_profile_section($pname, $psection, $i++);
         }
     }
+
     return;
 }
 
@@ -479,39 +603,36 @@ sub _clean_fields {
 # error reporting).
 sub _read_profile_section {
     my ($self, $pname, $section, $sno) = @_;
+
     my @tags = $self->_split_comma_sep_field($section->{'tags'});
     my $overridable
       = $self->_parse_boolean($section->{'overridable'}, -1, $pname, $sno);
-    my $severity = $section->{'severity'}//'';
-    my $noover = $self->{'non-overridable-tags'};
+    my $severity = $section->{'severity'} // EMPTY;
+
+    my $noover = $self->non_overridable_tags;
     $self->_check_for_invalid_fields($section, \%SEC_FIELDS, $pname,
         "section $sno");
-    croak(
-        join(q{ },
-            qq{Profile "$pname" is missing Tags field},
-            "(or it is empty) in section $sno")) unless @tags;
-    croak(
-        join(q{ },
-            qq{Profile "$pname" contains invalid severity},
-            qq{"$severity" in section $sno}))
 
-      if $severity
-      && (!$SEVERITIES{$severity} || $severity eq 'classification');
+    croak "Profile $pname is missing Tags field (or is empty) in section $sno"
+      unless @tags;
+
+    croak "Profile $pname contains invalid severity $severity in section $sno"
+      if ($severity && !$SEVERITIES{$severity})
+      || $severity eq 'classification';
 
     foreach my $tag (@tags) {
+
+        my $taginfo = $self->known_tags->{$tag};
         croak "Unknown check $tag in $pname (section $sno)"
-          unless $self->{'known-tags'}{$tag};
-        if ($severity) {
-            my $t = $self->{'known-tags'}{$tag};
-            if ($t->severity(1) eq 'classification') {
-                croak(
-                    join(q{ },
-                        qq{${tag} is a classification tag},
-                        q{and cannot not be assigned a severity},
-                        qq{(profile "$pname", section $sno)}));
-            }
-            $t->set_severity($severity);
-        }
+          unless defined $taginfo;
+
+        croak
+"Classification tag $tag cannot take a severity (profile $pname, section $sno"
+          if $taginfo->severity(1) eq 'classification';
+
+        $taginfo->effective_severity($severity)
+          if length $severity;
+
         if ($overridable != -1) {
             if ($overridable) {
                 delete $noover->{$tag};
@@ -520,6 +641,7 @@ sub _read_profile_section {
             }
         }
     }
+
     return;
 }
 
@@ -534,34 +656,39 @@ sub _read_profile_section {
 #  parents).
 sub _read_profile_tags{
     my ($self, $pname, $pheader) = @_;
+
     $self->_check_for_invalid_fields($pheader, \%MAIN_FIELDS, $pname,
         'profile header');
     $self->_check_duplicates($pname, $pheader, 'load-checks',
         'enable-tags-from-check', 'disable-tags-from-check');
     $self->_check_duplicates($pname, $pheader, 'enable-tags', 'disable-tags');
+
     my $tags_from_check_sub = sub {
         my ($field, $check) = @_;
 
-        unless (exists $self->{'check-scripts'}{$check}) {
-            $self->_load_check($pname, $check);
-        }
-        return $self->{'check-scripts'}{$check}->tags;
+        $self->_load_check($pname, $check)
+          unless exists $self->check_scripts->{$check};
+
+        return $self->check_scripts->{$check}->tags;
     };
+
     my $tag_sub = sub {
         my ($field, $tag) = @_;
-        unless (exists $self->{'known-tags'}{$tag}) {
-            croak "Unknown tag \"$tag\" in profile \"$pname\""
-              unless exists $self->{'known-tags'}{$tag};
-        }
+
+        croak "Unknown tag $tag in profile $pname"
+          unless exists $self->known_tags->{$tag};
+
         return $tag;
     };
+
     if ($pheader->{'load-checks'}) {
         for
           my $check ($self->_split_comma_sep_field($pheader->{'load-checks'})){
             $self->_load_check($pname, $check)
-              unless exists $self->{'check-scripts'}{$check};
+              unless exists $self->check_scripts->{$check};
         }
     }
+
     $self->_enable_tags_from_field($pname, $pheader, 'enable-tags-from-check',
         $tags_from_check_sub, 1);
     $self->_enable_tags_from_field($pname, $pheader, 'disable-tags-from-check',
@@ -570,6 +697,7 @@ sub _read_profile_tags{
         1);
     $self->_enable_tags_from_field($pname, $pheader, 'disable-tags', $tag_sub,
         0);
+
     return;
 }
 
@@ -580,13 +708,19 @@ sub _read_profile_tags{
 # these tags are enabled in the profile, otherwise they are disabled.
 sub _enable_tags_from_field {
     my ($self, $pname, $pheader, $field, $code, $enable) = @_;
+
     my $method = \&enable_tags;
-    my @tags;
-    $method = \&disable_tags unless $enable;
-    return unless $pheader->{$field};
-    @tags = map { $code->($field, $_) }
+
+    $method = \&disable_tags
+      unless $enable;
+
+    return
+      unless $pheader->{$field};
+
+    my @tags = map { $code->($field, $_) }
       $self->_split_comma_sep_field($pheader->{$field});
     $self->$method(@tags);
+
     return;
 }
 
@@ -601,25 +735,29 @@ sub _enable_tags_from_field {
 #
 sub _check_duplicates{
     my ($self, $name, $map, @fields) = @_;
+
     my %dupmap;
     foreach my $field (@fields) {
-        next unless exists $map->{$field};
-        foreach my $element (split m/\s*+,\s*+/o, $map->{$field}){
+
+        next
+          unless exists $map->{$field};
+
+        foreach my $element (split m/\s*+,\s*+/, $map->{$field}){
+
             if (exists $dupmap{$element}){
                 my $other = $dupmap{$element};
-                croak(
-                    join(q{ },
-                        qq{"$element" appears in both "$field"},
-                        qq{and "$other" in profile "$name"})
-                ) unless $other eq $field;
-                croak(
-                    join(q{ },
-                        qq{"$element" appears twice in the field},
-                        qq{"$field" in profile "$name"}));
+                croak
+                  "$element appears in both $field and $other in profile $name"
+                  unless $other eq $field;
+
+                croak
+"$element appears twice in the field $field in profile $name";
             }
+
             $dupmap{$element} = $field;
         }
     }
+
     return;
 }
 
@@ -630,11 +768,16 @@ sub _check_duplicates{
 # error reporting.
 sub _parse_boolean {
     my ($self, $bool, $def, $pname, $sno) = @_;
+
+    return $def
+      unless defined $bool;
+
     my $val;
-    return $def unless defined $bool;
     eval { $val = parse_boolean($bool); };
-    croak "\"$bool\" is not a boolean value in $pname (section $sno)"
+
+    croak "$bool is not a boolean value in $pname (section $sno)"
       if $@;
+
     return $val;
 }
 
@@ -644,8 +787,11 @@ sub _parse_boolean {
 # be ignored).
 sub _split_comma_sep_field {
     my ($self, $data) = @_;
-    return () unless defined $data;
-    return split m/\s*,\s*/o, strip($data);
+
+    return ()
+      unless defined $data;
+
+    return split m/\s*,\s*/, strip($data);
 }
 
 # $self->_check_for_invalid_fields($para, $known, $pname, $paraname)
@@ -655,15 +801,19 @@ sub _split_comma_sep_field {
 # to identify the profile name and paragraph (respectively)
 sub _check_for_invalid_fields {
     my ($self, $para, $known, $pname, $paraname) = @_;
+
     foreach my $field (keys %$para) {
-        next if exists $known->{$field};
-        croak "Unknown field \"$field\" in $pname ($paraname)";
+        next
+          if exists $known->{$field};
+        croak "Unknown field $field in $pname ($paraname)";
     }
+
     return;
 }
 
 sub _load_check {
     my ($self, $profile, $check) = @_;
+
     my $dir;
     foreach my $checkdir ($self->_safe_include_path('checks')) {
         my $cf = "$checkdir/${check}.desc";
@@ -681,29 +831,36 @@ sub _load_check {
 
 sub _parse_check {
     my ($self, $gcname, $dir) = @_;
+
     # Have we already tried to load this before?  Possibly via an alias
     # or symlink
-    return $self->{'check-scripts'}{$gcname}
-      if exists $self->{'check-scripts'}{$gcname};
-    my $c = Lintian::CheckScript->new($dir, $gcname);
+    return $self->check_scripts->{$gcname}
+      if exists $self->check_scripts->{$gcname};
+
+    my $c = Lintian::CheckScript->new;
+    $c->basedir($dir);
+    $c->name($gcname);
+    $c->load;
+
     my $cname = $c->name;
-    if (exists $self->{'check-scripts'}{$cname}) {
+    if (exists $self->check_scripts->{$cname}) {
         # We have loaded the check under a different name
-        $c = $self->{'check-scripts'}{$cname};
+        $c = $self->check_scripts->{$cname};
         # Record the alias so we don't have to parse the check file again.
-        $self->{'check-scripts'}{$gcname} = $c;
+        $self->check_scripts->{$gcname} = $c;
         return $c;
     }
-    $self->{'check-scripts'}{$cname} = $c;
-    $self->{'check-scripts'}{$gcname} = $c if $gcname ne $cname;
+    $self->check_scripts->{$cname} = $c;
+    $self->check_scripts->{$gcname} = $c
+      if $gcname ne $cname;
 
     die "Unknown check $gcname"
-      unless defined $self->{'check-tagnames'}{$gcname};
+      unless defined $self->check_tagnames->{$gcname};
 
-    my @tagnames = @{$self->{'check-tagnames'}{$gcname}};
+    my @tagnames = @{$self->check_tagnames->{$gcname}};
     for my $tagname (@tagnames) {
-        my $taginfo = $self->{'known-tags'}{$tagname};
-        $taginfo->add_script_type($c->type);
+        my $taginfo = $self->known_tags->{$tagname};
+        $taginfo->script_type($c->type);
 
         $c->add_taginfo($taginfo);
     }
@@ -725,17 +882,25 @@ sub _load_checks {
             $self->_parse_check($name, $checkdir);
         }
     }
+
     return;
 }
 
 sub _default_inc_path {
     my @path;
+
     push @path, "$ENV{'HOME'}/.lintian"
-      if exists $ENV{'HOME'} and defined $ENV{'HOME'};
+      if defined $ENV{'HOME'};
+
     push @path, '/etc/lintian';
+
     # ENV{LINTIAN_ROOT} replaces /usr/share/lintian if present.
-    push @path, $ENV{'LINTIAN_ROOT'} if defined $ENV{'LINTIAN_ROOT'};
-    push @path, '/usr/share/lintian' unless defined $ENV{'LINTIAN_ROOT'};
+    push @path, $ENV{'LINTIAN_ROOT'}
+      if defined $ENV{'LINTIAN_ROOT'};
+
+    push @path, '/usr/share/lintian'
+      unless defined $ENV{'LINTIAN_ROOT'};
+
     return @path;
 }
 
@@ -746,6 +911,7 @@ sub _find_vendor_profile {
     if (defined $prof and $prof !~ m/[{}]/) {
         # no substitution required...
         return ($self->_find_profile($prof), $prof);
+
     } elsif (defined $prof) {
         my $cpy = $prof;
         # Check for unknown (or broken) subst.
@@ -758,37 +924,247 @@ sub _find_vendor_profile {
 
     $prof //= '{VENDOR}/main';
 
-    @vendors = @{ $self->{'_vendor_cache'} }
-      if exists $self->{'_vendor_cache'};
+    @vendors = @{ $self->vendor_cache };
     unless (@vendors) {
+
         my $vendor = Dpkg::Vendor::get_current_vendor();
         croak 'Could not determine the current vendor'
           unless $vendor;
-        push @vendors, lc $vendor;
+
+        push(@vendors, lc $vendor);
         while ($vendor) {
             my $info = Dpkg::Vendor::get_vendor_info($vendor);
             # Cannot happen atm, but in case Dpkg::Vendor changes its internals
             #  or our code changes
             croak "Could not look up the parent vendor of $vendor"
               unless $info;
+
             $vendor = $info->{'Parent'};
-            push @vendors, lc $vendor if $vendor;
+            push(@vendors, lc $vendor)
+              if $vendor;
         }
-        $self->{'_vendor_cache'} = \@vendors;
+
+        $self->vendor_cache(\@vendors);
     }
+
     foreach my $vendor (@vendors) {
-        my $file;
+
         my $profname = $prof;
         $profname =~ s/\Q{VENDOR}\E/$vendor/g;
 
-        $file = $self->_find_profile($profname);
+        my $file = $self->_find_profile($profname);
 
-        return ($file, $profname) if $file;
+        return ($file, $profname)
+          if $file;
     }
-    croak(
-        join(q{ },
-            'Could not find a profile matching',
-            qq{"$prof" for vendor $vendors[0]}));
+
+    croak "Could not find a profile matching $prof for vendor $vendors[0]";
+}
+
+=item sources([SOURCE [, ...]])
+
+Limits the displayed tags to only those from the listed sources.  One or
+more sources may be given.  If no sources are given, resets the object
+ to display tags from any source.  Tag sources are the
+names of references from the Ref metadata for the tags.
+
+=cut
+
+sub sources {
+    my ($self, @sources) = @_;
+
+    $self->display_source({});
+
+    $self->display_source->{$_} = 1 for @sources;
+
+    return;
+}
+
+=item displayed(TAG)
+
+Returns true if the given tag would be displayed given the current
+configuration, false otherwise.  This does not check overrides, only whether
+the tag severity, certainty, and source warrants display given the
+configuration.
+
+=cut
+
+sub displayed {
+    my ($self, $tag) = @_;
+
+    # Note, we get the known as it will be suppressed by
+    # $self->suppressed below if the tag is not enabled.
+    my $info = $self->get_tag($tag, 1);
+    return 0
+      if ($info->experimental and not $self->{show_experimental});
+
+    return 0
+      if $self->suppressed($tag);
+
+    my $severity = $info->severity;
+    my $certainty = $info->certainty;
+
+    my $display = $self->display_level->{$severity}{$certainty};
+
+    # If display_source is set, we need to check whether any of the references
+    # of this tag occur in display_source.
+    if (keys %{ $self->display_source }) {
+        my @sources = $info->sources;
+        unless (any { $self->display_source->{$_} } @sources) {
+            $display = 0;
+        }
+    }
+
+    return $display;
+}
+
+=item suppressed(TAG)
+
+Returns true if the given tag would be suppressed given the current
+configuration, false otherwise.  This is different than displayed() in
+that a tag is only suppressed if Lintian treats the tag as if it's never
+been seen, doesn't update statistics, and doesn't change its exit status.
+Tags are suppressed via profile().
+
+=cut
+
+sub suppressed {
+    my ($self, $tag) = @_;
+
+    return 1
+      unless $self->get_tag($tag);
+
+    return;
+}
+
+=item display(OPERATION, RELATION, SEVERITY, CERTAINTY)
+
+Configure which tags are displayed by severity and certainty.  OPERATION
+is C<+> to display the indicated tags, C<-> to not display the indicated
+tags, or C<=> to not display any tags except the indicated ones.  RELATION
+is one of C<< < >>, C<< <= >>, C<=>, C<< >= >>, or C<< > >>.  The
+OPERATION will be applied to all pairs of severity and certainty that
+match the given RELATION on the SEVERITY and CERTAINTY arguments.  If
+either of those arguments are undefined, the action applies to any value
+for that variable.  For example:
+
+    $tags->display('=', '>=', 'important');
+
+turns off display of all tags and then enables display of any tag (with
+any certainty) of severity important or higher.
+
+    $tags->display('+', '>', 'normal', 'possible');
+
+adds to the current configuration display of all tags with a severity
+higher than normal and a certainty higher than possible (so
+important/certain and serious/certain).
+
+    $tags->display('-', '=', 'minor', 'possible');
+
+turns off display of tags of severity minor and certainty possible.
+
+This method throws an exception on errors, such as an unknown severity or
+certainty or an impossible constraint (like C<< > serious >>).
+
+=cut
+
+# Generate a subset of a list given the element and the relation.  This
+# function makes a hard assumption that $rel will be one of <, <=, =, >=,
+# or >.  It is not syntax-checked.
+sub _relation_subset {
+    my ($self, $element, $rel, @list) = @_;
+
+    if ($rel eq '=') {
+        return grep { $_ eq $element } @list;
+    }
+
+    if (substr($rel, 0, 1) eq '<') {
+        @list = reverse @list;
+    }
+
+    my $found;
+    for my $i (0..$#list) {
+        if ($element eq $list[$i]) {
+            $found = $i;
+            last;
+        }
+    }
+
+    return
+      unless defined($found);
+
+    if (length($rel) > 1) {
+        return @list[$found .. $#list];
+
+    }
+
+    return
+      if $found == $#list;
+
+    return @list[($found + 1) .. $#list];
+}
+
+# Given the operation, relation, severity, and certainty, produce a
+# human-readable representation of the display level string for errors.
+sub _format_level {
+    my ($self, $op, $rel, $severity, $certainty) = @_;
+
+    if (not defined $severity and not defined $certainty) {
+        return "$op $rel";
+    } elsif (not defined $severity) {
+        return "$op $rel $certainty (certainty)";
+    } elsif (not defined $certainty) {
+        return "$op $rel $severity (severity)";
+    } else {
+        return "$op $rel $severity/$certainty";
+    }
+}
+
+sub display {
+    my ($self, $op, $rel, $severity, $certainty) = @_;
+
+    unless ($op =~ /^[+=-]\z/ and $rel =~ /^(?:[<>]=?|=)\z/) {
+        my $error = $self->_format_level($op, $rel, $severity, $certainty);
+        die 'invalid display constraint ' . $error;
+    }
+
+    if ($op eq '=') {
+        for my $s (@Lintian::Tag::Info::SEVERITIES) {
+            for my $c (@Lintian::Tag::Info::CERTAINTIES) {
+                $self->display_level->{$s}{$c} = 0;
+            }
+        }
+    }
+
+    my $status = ($op eq '-' ? 0 : 1);
+
+    my (@severities, @certainties);
+    if ($severity) {
+        @severities = $self->_relation_subset($severity, $rel,
+            @Lintian::Tag::Info::SEVERITIES);
+    } else {
+        @severities = @Lintian::Tag::Info::SEVERITIES;
+    }
+
+    if ($certainty) {
+        @certainties = $self->_relation_subset($certainty, $rel,
+            @Lintian::Tag::Info::CERTAINTIES);
+    } else {
+        @certainties = @Lintian::Tag::Info::CERTAINTIES;
+    }
+
+    unless (@severities and @certainties) {
+        my $error = $self->_format_level($op, $rel, $severity, $certainty);
+        die 'invalid display constraint ' . $error;
+    }
+
+    for my $s (@severities) {
+        for my $c (@certainties) {
+            $self->{display_level}{$s}{$c} = $status;
+        }
+    }
+
+    return;
 }
 
 =back
